import { system, BlockPermutation, world } from "@minecraft/server";
import { ActionFormData } from "@minecraft/server-ui";

const ui = new ActionFormData()
ui.title("Settings")
ui.body("")
ui.button("Impact frames (Fixes Crashes)")
ui.button("Block Destruction/Griefing")
ui.button("Toggle Cooldowns");

world.afterEvents.itemUse.subscribe(event => {
  const { source, itemStack } = event;

  if (itemStack.typeId === 'ds:settings') {
    ui.show(source).then(result => {
      if (result.selection === 0) source.runCommand('function setting_impactframes');
      if (result.selection === 1) source.runCommand('function setting_griefing');
      if (result.selection === 2) source.runCommand('function setting_nocds');
    }).catch(error => {
      console.error("Error showing UI: ", error);
    });
  }
});
    
const processedThisTick = new Set();

world.beforeEvents.itemUse.subscribe(({ itemStack, source }) => {
  const itemId = `itemUse:${itemStack.typeId}`;
  if (!processedThisTick.has(itemId)) {
    system.run(() => {
      source.addTag(itemId);
    });
    system.runTimeout(() => {
      source.removeTag(itemId);
      processedThisTick.delete(itemId);
    }, 1); // Adjusted timeout to 1 tick
    processedThisTick.add(itemId);
  }
});

system.afterEvents.scriptEventReceive.subscribe((event) => {
  var player = event.sourceEntity;
  const di = player.getViewDirection();
  switch (event.id) {
    case "sl:knockback0.3":
      player.applyKnockback(di.x, di.z, -0.3, 0);
      break;
    case "sl:forwardrotate":
        player.applyKnockback(di.x, di.z, 1, 0);
       break;
    case "sl:forward0.2":
        player.applyKnockback(di.x, di.z, 0.18, 0);
        break;
    case "sl:forward0.3":
      player.applyKnockback(di.x, di.z, 0.3, 0);
      break;
    case "sl:forward0.22":
      player.applyKnockback(di.x, di.z, 0.22, 0);
      break;
    case "sl:forward12":
      player.applyKnockback(di.x, di.z, 12, 0);
      break;
    case "sl:forwarddown1":
      player.applyKnockback(di.x, di.z, 1, -0.5);
       break;
    case "sl:forward1":
      player.applyKnockback(di.x, di.z, 1, 0);
      break;
    case "sl:forward1.5":
       player.applyKnockback(di.x, di.z, 1.5, 0);
      break; 
    case "sl:forward2":
      player.applyKnockback(di.x, di.z, 2, 0);
      break;
    case "sl:forward3":
      player.applyKnockback(di.x, di.z, 3, 0);
      break;
    case "sl:forward4":
      player.applyKnockback(di.x, di.z, 4, 0);
      break;
    case "sl:forward5":
      player.applyKnockback(di.x, di.z, 5, 0);
      break;
    case "sl:backward12":
      player.applyKnockback(di.x, di.z, -7, 0);
      break;
    case "sl:backward3":
        player.applyKnockback(di.x, di.z, -3.5, 0);
      break;
    case "sl:backward4":
        player.applyKnockback(di.x, di.z, -7, 0);
      break;
    case "sl:backward1":
        player.applyKnockback(di.x, di.z, -1, 0);
      break;
    case "sl:dashEast":
      player.applyKnockback(di.z, -di.x, -9, 0);
      break;
    case "sl:dashWest":
      player.applyKnockback(di.z, -di.x, 9, 0);
      break;
    case "sl:antiair":
      const anti_velo1 = player.getVelocity();
      let anti_velocity1 = anti_velo1.y * -1
      player.applyKnockback(di.x, di.z, 0.2, anti_velocity1);
      break;
  }
});

// Movement direction is turned in 0-359 degrees; 0-North 90-East 180-South 270-West (Give a little wiggle room like for north detect if the angle is either 350-359 or 0-10) Scoreboard is "moveAngle" youll most likely need to create this objective
function moveAngleDetect() {
  system.runInterval(() => {
    for (const player of world.getPlayers()) {
      const velocity = player.getVelocity();
      const di = player.getViewDirection();
      let moveAngle = Math.atan2(velocity.z, velocity.x) - Math.atan2(di.z, di.x);
      moveAngle = (moveAngle * (180 / Math.PI) + 360) % 360;
      moveAngle = Math.round(moveAngle)
      player.runCommand(`scoreboard players set @s moveAngle ${moveAngle}`);
    }
  });
}
moveAngleDetect();








